@@ -3,7 +3,7 @@
 
 A bitmask enum attribute macro, to turn an enum into a bitmask.
 
-A bitmask can have unsigned integer types, the default type is `usize`.
+A bitmask can have (un)signed integer types, the default type is `usize`.
 
 ```
 use bitmask_enum::bitmask;
@@ -27,6 +27,12 @@ enum Bitmask {
     Flag3, // defaults to 0d00000100
 }
 
+impl Bitmask {
+    fn f1_or_f3(&self) -> bool {
+        self.contains(Self::Flag1 | Self::Flag3)
+    }
+}
+
 // bitmask has const bitwise operator methods
 const CONST_BM: Bitmask = Bitmask::Flag2.or(Bitmask::Flag3);
 
@@ -42,43 +48,47 @@ println!("{}", bm.intersects(CONST_BM)); // true
 
 // Does bm contain all of CONST_BM
 println!("{}", bm.contains(CONST_BM)); // false
+
+// Does bm contain all of CONST_BM
+println!("{}", bm.f1_or_f3()); // true
 ```
 
 ## Custom Values
 
 You can assign every flag a custom value.
 
-Because behind the scences `enum Bitmask` gets converted to a `struct Bitmask(u8);` you need to wrap `u8` expressions into a `Self(_)`.
-
 ```
 use bitmask_enum::bitmask;
 
 #[bitmask(u8)]
 enum Bitmask {
-    Flag1 = Self(0b00010000),
-    Flag2 = Self(0b00000100),
-    Flag3 = Self(0b00000001),
+    Flag5 = 0b00010000,
+    Flag3 = 0b00000100,
+    Flag1 = 0b00000001,
 
-    Flag13_1 = Self(0b00010000 | 0b00000001),
-    Flag13_2 = Self::Flag1.or(Self::Flag3),
+    Flag51_1 = 0b00010000 | 0b00000001,
+    Flag51_2 = Self::Flag5.or(Self::Flag1).bits,
+    Flag51_3 = Self::Flag5.bits | Self::Flag1.bits,
 
-    Flag4 = Self({
-        let left = Self::Flag13_1;
-        left.0 | Self::Flag2.0
-    }),
+    Flag513 = {
+        let flag51 = Self::Flag51_1.bits;
+        flag51 | Self::Flag3.bits
+    },
 }
 
-let bm = Bitmask::Flag1 | Bitmask::Flag3;
+let bm = Bitmask::Flag5 | Bitmask::Flag1;
 
 println!("{:#010b}", bm); // 0b00010001
-println!("{}", bm == Bitmask::Flag13_1); // true
-println!("{}", bm == Bitmask::Flag13_2); // true
+println!("{}", bm == Bitmask::Flag51_1); // true
 
-println!("{:#010b}", Bitmask::Flag4); // 0b00010101
+println!("{:#010b}", Bitmask::Flag513); // 0b00010101
 ```
 
 ## Implemented Methods
 ```ignore
+// returns the underlying bits
+const fn bits(&self) -> #type {
+
 // contains all values
 const fn all() -> Self;
 
@@ -108,7 +118,7 @@ const fn xor(self, other: Self) -> Self;
 
 ## Implemented Traits
 ```ignore
-#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash)]
 
 impl core::ops::Not;
 
@@ -127,6 +137,9 @@ impl From<#ident> for #type;
 impl PartialEq<#type>;
 
 impl core::fmt::Binary;
+impl core::fmt::LowerHex;
+impl core::fmt::UpperHex;
+impl core::fmt::Octal;
 ```
 */
 
@@ -157,73 +170,81 @@ pub fn bitmask(attr: TokenStream, item: TokenStream) -> TokenStream {
     let (ident, idents, exprs) = enm(item);
 
     let enm = quote::quote! {
-        #[derive(Debug, Clone, Copy, PartialEq, Eq)]
-        #vis struct #ident(#typ);
+        #[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash)]
+        #vis struct #ident {
+            bits: #typ,
+        }
 
         #[allow(non_upper_case_globals)]
         impl #ident {
             #(#vis const #idents: #ident = #exprs;)*
 
+            /// returns the underlying bits
+            #[inline]
+            #vis const fn bits(&self) -> #typ {
+                self.bits
+            }
+
             /// contains all values
             #[inline]
             #vis const fn all() -> Self {
-                Self(!0)
+                Self { bits: !0 }
             }
 
             /// if self contains all values
             #[inline]
             #vis const fn is_all(&self) -> bool {
-                self.0 == !0
+                self.bits == !0
             }
 
             /// contains no value
             #[inline]
             #vis const fn none() -> Self {
-                Self(0)
+                Self { bits: 0 }
             }
 
             /// if self contains no value
             #[inline]
             #vis const fn is_none(&self) -> bool {
-                self.0 == 0
+                self.bits == 0
             }
 
             /// self intersects one of the other
             /// `(self & other) != 0 || other == 0`
             #[inline]
             #vis const fn intersects(&self, other: Self) -> bool {
-                (self.0 & other.0) != 0 || other.0 == 0
+                (self.bits & other.bits) != 0 || other.bits == 0
             }
 
             /// self contains all of the other
             /// `(self & other) == other`
             #[inline]
             #vis const fn contains(&self, other: Self) -> bool {
-                (self.0 & other.0) == other.0
+                (self.bits & other.bits) == other.bits
             }
 
             /// constant bitwise not
             #[inline]
             #vis const fn not(self) -> Self {
-                Self(!self.0)
+                Self { bits: !self.bits }
             }
 
             /// constant bitwise and
             #[inline]
             #vis const fn and(self, other: Self) -> Self {
-                Self(self.0 & other.0)
+                Self { bits: self.bits & other.bits }
             }
 
             /// constant bitwise or
             #[inline]
             #vis const fn or(self, other: Self) -> Self {
-                Self(self.0 | other.0)
+                Self { bits: self.bits | other.bits }
             }
 
             /// constant bitwise xor
             #[inline]
             #vis const fn xor(self, other: Self) -> Self {
-                Self(self.0 ^ other.0)
+                Self { bits: self.bits ^ other.bits }
             }
         }
 
@@ -231,7 +252,7 @@ pub fn bitmask(attr: TokenStream, item: TokenStream) -> TokenStream {
             type Output = Self;
             #[inline]
             fn not(self) -> Self::Output {
-                Self(self.0.not())
+                Self { bits: self.bits.not() }
             }
         }
 
@@ -239,14 +260,14 @@ pub fn bitmask(attr: TokenStream, item: TokenStream) -> TokenStream {
             type Output = Self;
             #[inline]
             fn bitand(self, rhs: Self) -> Self::Output {
-                Self(self.0.bitand(rhs.0))
+                Self { bits: self.bits.bitand(rhs.bits) }
             }
         }
 
         impl core::ops::BitAndAssign for #ident {
             #[inline]
             fn bitand_assign(&mut self, rhs: Self){
-                self.0.bitand_assign(rhs.0)
+                self.bits.bitand_assign(rhs.bits)
             }
         }
 
@@ -254,14 +275,14 @@ pub fn bitmask(attr: TokenStream, item: TokenStream) -> TokenStream {
             type Output = Self;
             #[inline]
             fn bitor(self, rhs: Self) -> Self::Output {
-                Self(self.0.bitor(rhs.0))
+                Self { bits: self.bits.bitor(rhs.bits) }
             }
         }
 
         impl core::ops::BitOrAssign for #ident {
             #[inline]
             fn bitor_assign(&mut self, rhs: Self){
-                self.0.bitor_assign(rhs.0)
+                self.bits.bitor_assign(rhs.bits)
             }
         }
 
@@ -269,41 +290,59 @@ pub fn bitmask(attr: TokenStream, item: TokenStream) -> TokenStream {
             type Output = Self;
             #[inline]
             fn bitxor(self, rhs: Self) -> Self::Output {
-                Self(self.0.bitxor(rhs.0))
+                Self { bits: self.bits.bitxor(rhs.bits) }
             }
         }
 
         impl core::ops::BitXorAssign for #ident {
             #[inline]
             fn bitxor_assign(&mut self, rhs: Self){
-                self.0.bitxor_assign(rhs.0)
+                self.bits.bitxor_assign(rhs.bits)
             }
         }
 
         impl From<#typ> for #ident {
             #[inline]
             fn from(val: #typ) -> Self {
-                Self(val)
+                Self { bits: val }
             }
         }
 
         impl From<#ident> for #typ {
             #[inline]
             fn from(val: #ident) -> #typ {
-                val.0
+                val.bits
             }
         }
 
         impl PartialEq<#typ> for #ident {
             #[inline]
             fn eq(&self, other: &#typ) -> bool {
-                self.0 == *other
+                self.bits == *other
             }
         }
 
         impl core::fmt::Binary for #ident {
             fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
-                self.0.fmt(f)
+                self.bits.fmt(f)
+            }
+        }
+
+        impl core::fmt::LowerHex for #ident {
+            fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
+                self.bits.fmt(f)
+            }
+        }
+
+        impl core::fmt::UpperHex for #ident {
+            fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
+                self.bits.fmt(f)
+            }
+        }
+
+        impl core::fmt::Octal for #ident {
+            fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
+                self.bits.fmt(f)
             }
         }
     };
@@ -318,7 +357,8 @@ fn typ(attr: TokenStream) -> String {
         let typ = attr.into_iter().next().expect("unreachable").to_string();
         match typ.as_str() {
             "u8" | "u16" | "u32" | "u64" | "u128" | "usize" => (),
-            _ => panic!("type can only be an unsigned integer."),
+            "i8" | "i16" | "i32" | "i64" | "i128" | "isize" => (),
+            _ => panic!("type can only be an (un)signed integer."),
         }
         typ
     }
@@ -347,9 +387,9 @@ fn enm(item: ItemEnum) -> (Ident, Vec<Ident>, Vec<impl quote::ToTokens>) {
 
         if hv {
             let (_, ref expr) = v.discriminant.as_ref().expect("unreachable");
-            exprs.push(quote::quote!(#expr));
+            exprs.push(quote::quote!(Self { bits: #expr }));
         } else {
-            exprs.push(quote::quote!(#ident(1 << #i)));
+            exprs.push(quote::quote!(Self { bits: 1 << #i }));
         }
     });
 
